package sun.security.krb5.internal.ktab;

import sun.security.krb5.internal.Krb5;
import java.util.HashMap;
import java.io.FileOutputStream;
import sun.security.krb5.internal.KerberosTime;
import java.util.Arrays;
import java.util.Comparator;
import sun.security.krb5.internal.crypto.EType;
import java.util.ArrayList;
import sun.security.krb5.EncryptionKey;
import sun.security.krb5.PrincipalName;
import sun.security.krb5.RealmException;
import java.io.IOException;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import sun.security.krb5.KrbException;
import java.util.StringTokenizer;
import sun.security.krb5.Config;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.File;
import java.util.Vector;
import java.util.Map;

public class KeyTab implements KeyTabConstants
{
    private static final boolean DEBUG;
    private static String defaultTabName;
    private static Map<String, KeyTab> map;
    private boolean isMissing;
    private boolean isValid;
    private final String tabName;
    private long lastModified;
    private int kt_vno;
    private Vector<KeyTabEntry> entries;
    
    private KeyTab(String s) {
        this.isMissing = false;
        this.isValid = true;
        this.kt_vno = 1282;
        this.entries = new Vector();
        this.tabName = s;
        try {
            this.lastModified = new File(this.tabName).lastModified();
            try (final KeyTabInputStream keyTabInputStream = new KeyTabInputStream(new FileInputStream(s))) {
                this.load(keyTabInputStream);
            }
        }
        catch (FileNotFoundException ex) {
            this.entries.clear();
            this.isMissing = true;
        }
        catch (Exception ex2) {
            this.entries.clear();
            this.isValid = false;
        }
    }
    
    private static synchronized KeyTab getInstance0(String pathname) {
        long lastModified = new File(pathname).lastModified();
        KeyTab keyTab = (KeyTab)KeyTab.map.get(pathname);
        if (keyTab != null && keyTab.isValid() && keyTab.lastModified == lastModified) {
            return keyTab;
        }
        KeyTab keyTab2 = new KeyTab(pathname);
        if (keyTab2.isValid()) {
            KeyTab.map.put(pathname, keyTab2);
            return keyTab2;
        }
        if (keyTab != null) {
            return keyTab;
        }
        return keyTab2;
    }
    
    public static KeyTab getInstance(String s) {
        if (s == null) {
            return getInstance();
        }
        return getInstance0(normalize(s));
    }
    
    public static KeyTab getInstance(File file) {
        if (file == null) {
            return getInstance();
        }
        return getInstance0(file.getPath());
    }
    
    public static KeyTab getInstance() {
        return getInstance(getDefaultTabName());
    }
    
    public boolean isMissing() {
        return this.isMissing;
    }
    
    public boolean isValid() {
        return this.isValid;
    }
    
    private static String getDefaultTabName() {
        if (KeyTab.defaultTabName != null) {
            return KeyTab.defaultTabName;
        }
        String s = null;
        try {
            String value = Config.getInstance().get(new String[] { "libdefaults", "default_keytab_name" });
            if (value != null) {
                StringTokenizer stringTokenizer = new StringTokenizer(value, " ");
                while (stringTokenizer.hasMoreTokens()) {
                    s = normalize(stringTokenizer.nextToken());
                    if (new File(s).exists()) {
                        break;
                    }
                    continue;
                }
            }
        }
        catch (KrbException ex) {
            s = null;
        }
        if (s == null) {
            String str = (String)AccessController.doPrivileged(new GetPropertyAction("user.home"));
            if (str == null) {
                str = (String)AccessController.doPrivileged(new GetPropertyAction("user.dir"));
            }
            s = str + File.separator + "krb5.keytab";
        }
        return KeyTab.defaultTabName = s;
    }
    
    public static String normalize(String s) {
        String s2;
        if (s.length() >= 5 && s.substring(0, 5).equalsIgnoreCase("FILE:")) {
            s2 = s.substring(5);
        }
        else if (s.length() >= 9 && s.substring(0, 9).equalsIgnoreCase("ANY:FILE:")) {
            s2 = s.substring(9);
        }
        else if (s.length() >= 7 && s.substring(0, 7).equalsIgnoreCase("SRVTAB:")) {
            s2 = s.substring(7);
        }
        else {
            s2 = s;
        }
        return s2;
    }
    
    private void load(KeyTabInputStream keyTabInputStream) throws IOException, RealmException {
        this.entries.clear();
        this.kt_vno = keyTabInputStream.readVersion();
        if (this.kt_vno == 1281) {
            keyTabInputStream.setNativeByteOrder();
        }
        while (keyTabInputStream.available() > 0) {
            int entryLength = keyTabInputStream.readEntryLength();
            KeyTabEntry entry = keyTabInputStream.readEntry(entryLength, this.kt_vno);
            if (KeyTab.DEBUG) {
                System.out.println(">>> KeyTab: load() entry length: " + entryLength + "; type: " + ((entry != null) ? entry.keyType : 0));
            }
            if (entry == null) {
                continue;
            }
            this.entries.addElement(entry);
        }
    }
    
    public PrincipalName getOneName() {
        int size = this.entries.size();
        return (size > 0) ? ((KeyTabEntry)this.entries.elementAt(size - 1)).service : null;
    }
    
    public EncryptionKey[] readServiceKeys(PrincipalName obj) {
        int size = this.entries.size();
        ArrayList<EncryptionKey> list = new ArrayList(size);
        if (KeyTab.DEBUG) {
            System.out.println("Looking for keys for: " + obj);
        }
        for (int i = size - 1; i >= 0; --i) {
            KeyTabEntry keyTabEntry = (KeyTabEntry)this.entries.elementAt(i);
            if (keyTabEntry.service.match(obj)) {
                if (EType.isSupported(keyTabEntry.keyType)) {
                    list.add(new EncryptionKey(keyTabEntry.keyblock, keyTabEntry.keyType, new Integer(keyTabEntry.keyVersion)));
                    if (KeyTab.DEBUG) {
                        System.out.println("Added key: " + keyTabEntry.keyType + "version: " + keyTabEntry.keyVersion);
                    }
                }
                else if (KeyTab.DEBUG) {
                    System.out.println("Found unsupported keytype (" + keyTabEntry.keyType + ") for " + obj);
                }
            }
        }
        EncryptionKey[] a = (EncryptionKey[])list.toArray(new EncryptionKey[list.size()]);
        Arrays.sort(a, new Comparator<EncryptionKey>() {
            @Override
            public int compare(EncryptionKey encryptionKey, EncryptionKey encryptionKey2) {
                return (int)encryptionKey2.getKeyVersionNumber() - (int)encryptionKey.getKeyVersionNumber();
            }
        });
        return a;
    }
    
    public boolean findServiceEntry(PrincipalName principalName) {
        for (int i = 0; i < this.entries.size(); ++i) {
            KeyTabEntry keyTabEntry = (KeyTabEntry)this.entries.elementAt(i);
            if (keyTabEntry.service.match(principalName)) {
                if (EType.isSupported(keyTabEntry.keyType)) {
                    return true;
                }
                if (KeyTab.DEBUG) {
                    System.out.println("Found unsupported keytype (" + keyTabEntry.keyType + ") for " + principalName);
                }
            }
        }
        return false;
    }
    
    public String tabName() {
        return this.tabName;
    }
    
    public void addEntry(PrincipalName principalName, char[] array, int n, boolean b) throws KrbException {
        this.addEntry(principalName, principalName.getSalt(), array, n, b);
    }
    
    public void addEntry(PrincipalName pname, String salt, char[] password, int n, boolean b) throws KrbException {
        EncryptionKey[] acquireSecretKeys = EncryptionKey.acquireSecretKeys(password, salt);
        int keyVersion = 0;
        for (int i = this.entries.size() - 1; i >= 0; --i) {
            KeyTabEntry keyTabEntry = (KeyTabEntry)this.entries.get(i);
            if (keyTabEntry.service.match(pname)) {
                if (keyTabEntry.keyVersion > keyVersion) {
                    keyVersion = keyTabEntry.keyVersion;
                }
                if (!b || keyTabEntry.keyVersion == n) {
                    this.entries.removeElementAt(i);
                }
            }
        }
        if (n == -1) {
            n = keyVersion + 1;
        }
        for (int n2 = 0; acquireSecretKeys != null && n2 < acquireSecretKeys.length; ++n2) {
            this.entries.addElement(new KeyTabEntry(pname, pname.getRealm(), new KerberosTime(System.currentTimeMillis()), n, acquireSecretKeys[n2].getEType(), acquireSecretKeys[n2].getBytes()));
        }
    }
    
    public KeyTabEntry[] getEntries() {
        KeyTabEntry[] array = new KeyTabEntry[this.entries.size()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (KeyTabEntry)this.entries.elementAt(i);
        }
        return array;
    }
    
    public static synchronized KeyTab create() throws IOException, RealmException {
        return create(getDefaultTabName());
    }
    
    public static synchronized KeyTab create(String name) throws IOException, RealmException {
        try (final KeyTabOutputStream keyTabOutputStream = new KeyTabOutputStream(new FileOutputStream(name))) {
            keyTabOutputStream.writeVersion(1282);
        }
        return new KeyTab(name);
    }
    
    public synchronized void save() throws IOException {
        try (final KeyTabOutputStream keyTabOutputStream = new KeyTabOutputStream(new FileOutputStream(this.tabName))) {
            keyTabOutputStream.writeVersion(this.kt_vno);
            for (int i = 0; i < this.entries.size(); ++i) {
                keyTabOutputStream.writeEntry((KeyTabEntry)this.entries.elementAt(i));
            }
        }
    }
    
    public int deleteEntries(PrincipalName principalName, int n, int n2) {
        int n3 = 0;
        HashMap<Integer, Integer> hashMap = new HashMap();
        for (int i = this.entries.size() - 1; i >= 0; --i) {
            KeyTabEntry keyTabEntry = (KeyTabEntry)this.entries.get(i);
            if (principalName.match(keyTabEntry.getService()) && (n == -1 || keyTabEntry.keyType == n)) {
                if (n2 == -2) {
                    if (hashMap.containsKey(Integer.valueOf(keyTabEntry.keyType))) {
                        if (keyTabEntry.keyVersion > (int)hashMap.get(Integer.valueOf(keyTabEntry.keyType))) {
                            hashMap.put(Integer.valueOf(keyTabEntry.keyType), Integer.valueOf(keyTabEntry.keyVersion));
                        }
                    }
                    else {
                        hashMap.put(Integer.valueOf(keyTabEntry.keyType), Integer.valueOf(keyTabEntry.keyVersion));
                    }
                }
                else if (n2 == -1 || keyTabEntry.keyVersion == n2) {
                    this.entries.removeElementAt(i);
                    ++n3;
                }
            }
        }
        if (n2 == -2) {
            for (int j = this.entries.size() - 1; j >= 0; --j) {
                KeyTabEntry keyTabEntry2 = (KeyTabEntry)this.entries.get(j);
                if (principalName.match(keyTabEntry2.getService()) && (n == -1 || keyTabEntry2.keyType == n) && keyTabEntry2.keyVersion != (int)hashMap.get(Integer.valueOf(keyTabEntry2.keyType))) {
                    this.entries.removeElementAt(j);
                    ++n3;
                }
            }
        }
        return n3;
    }
    
    public synchronized void createVersion(File file) throws IOException {
        try (final KeyTabOutputStream keyTabOutputStream = new KeyTabOutputStream(new FileOutputStream(file))) {
            keyTabOutputStream.write16(1282);
        }
    }
    
    static {
        DEBUG = Krb5.DEBUG;
        KeyTab.defaultTabName = null;
        KeyTab.map = new HashMap();
    }
}
